//
// Copyright (c) 2012, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Windows.Forms;

using MindFusion.Diagramming.Layout;
using MindFusion.Drawing;


namespace MindFusion.Diagramming.WinForms.Samples.CS.TreeLayoutSample
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

            diagram.BackBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(224, 233, 233));
			diagram.DiagramStyle.ShadowBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(30, Color.Black));
			diagram.NodeEffects.Add(new GlassEffect());
		}

		private void pb_MouseDown(object sender, MouseEventArgs e)
		{
			PictureBox pb = (PictureBox)sender;
			DoDragDrop(pb.Image, DragDropEffects.Copy);
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			// create the root of the tree
			rootNode = new ShapeNode(diagram);
			rootNode.Bounds = new RectangleF(80, 15, 18, 10);
			rootNode.Text = "right-click me!";
            rootNode.Brush = new MindFusion.Drawing.LinearGradientBrush(Color.FromArgb(224, 233, 233),
                Color.FromArgb(102, 154, 204));
			rootNode.Tag = 0;
			diagram.Nodes.Add(rootNode);
		}

		private ShapeNode addChild(ShapeNode node)
		{
			// create the new node and add it to the parent group
			// so its position is updated when the parent is moved
			ShapeNode childNode = new ShapeNode(diagram);
			childNode.Bounds = new RectangleF(0, 0, 18, 10);
			childNode.AttachTo(node, AttachToNode.TopLeft);
			diagram.Nodes.Add(childNode);
            childNode.Brush = new MindFusion.Drawing.LinearGradientBrush(Color.FromArgb(224, 233, 233),
                Color.FromArgb(102, 154, 204));

			// link the parent node with the child
			DiagramLink link = new DiagramLink(diagram, node, childNode);
            link.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(102, 154, 204));
			diagram.Links.Add(link);

			// update the counter
			uniqueID += 1;
			childNode.Tag = uniqueID;

			// rearrange the tree
			Rearrange();

			// select the new node
			diagram.Selection.Change(childNode);

			return childNode;
		}

		private void Rearrange()
		{
			if (treeLayout == null)
			{
				treeLayout = new TreeLayout(rootNode,
					TreeLayoutType.Centered,
					false,
					TreeLayoutLinkType.Rounded,
					TreeLayoutDirections.TopToBottom,
					10, 5, true, new SizeF(10, 10));
			}

			treeLayout.Arrange(diagram);
		}

		private void menuItem1_Click(object sender, EventArgs e)
		{
			if (currNode != null)
				addChild(currNode);
			currNode = null;
		}

		private void diagramView_DragDrop(object sender, DragEventArgs e)
		{
			// we have the screen point of the mouse, but need it in document C.S.
			Point clientPt = diagramView.PointToClient(new Point(e.X, e.Y));
			PointF docPt = diagramView.ClientToDoc(clientPt);

			// allow drag and drop if there is a box
			ShapeNode box = diagram.GetNodeAt(docPt) as ShapeNode;
			if (box != null)
			{
				// add a child to the tree
				ShapeNode newNode = addChild(box);

				// show the node tag
				newNode.Text = "id: " + newNode.Tag.ToString();

				// if an image has been dropped
				if (e.Data.GetDataPresent(DataFormats.Bitmap))
				{
					// set as a node icon
					newNode.ImageAlign = ImageAlign.TopLeft;
					newNode.Image = (Image)e.Data.GetData(DataFormats.Bitmap);

					// move text to the right
					StringFormat rightAlmt = new StringFormat();
					rightAlmt.Alignment = StringAlignment.Far;
					rightAlmt.LineAlignment = StringAlignment.Center;
					newNode.TextFormat = new StringFormat(rightAlmt);
				}
			}
		}

		private void diagram_TreeCollapsed(object sender, NodeEventArgs e)
		{
			Rearrange();
		}

		private void diagram_TreeExpanded(object sender, NodeEventArgs e)
		{
			Rearrange();
		}

		private void diagram_NodeCreated(object sender, NodeEventArgs e)
		{
			RectangleF rc = e.Node.Bounds;
			diagram.Nodes.Remove(e.Node);

			ShapeNode child = addChild(rootNode);
			child.Resize(rc.Width, rc.Height);
			child.Text = "right-click me!";
		}

		private void diagramView_DragOver(object sender, DragEventArgs e)
		{
			// we have the screen point of the mouse, but need it in document C.S.
			Point clientPt = diagramView.PointToClient(new Point(e.X, e.Y));
			PointF docPt = diagramView.ClientToDoc(clientPt);

			// allow drag and drop if there is a box
			ShapeNode box = diagram.GetNodeAt(docPt) as ShapeNode;
			if (box != null)
				e.Effect = DragDropEffects.Copy;
			else
				e.Effect = DragDropEffects.None;
		}

		private void diagram_NodeClicked(object sender, NodeEventArgs e)
		{
			// display context menu on right-click
			if (e.MouseButton == MouseButton.Right)
			{
				currNode = e.Node as ShapeNode;
				cmNode.Show(this, PointToClient(MousePosition));
			}
		}

		private void diagram_NodeDeleting(object sender, NodeValidationEventArgs e)
		{
			if (e.Node == rootNode)
				e.Cancel = true;
		}

		private void diagram_LinkCreating(object sender, LinkValidationEventArgs e)
		{
			e.Cancel = true;
		}

		private void diagram_LinkModifying(object sender, LinkValidationEventArgs e)
		{
			e.Cancel = true;
		}

		private void diagram_LinkSelecting(object sender, LinkValidationEventArgs e)
		{
			e.Cancel = true;
		}

		private int uniqueID = 0;
		private TreeLayout treeLayout = null;
		private ShapeNode rootNode = null;
		private ShapeNode currNode = null;
	}
}